import 'package:ambito/src/entity/_general/filter/item_filter_repository.dart';
import 'package:ambito/src/entity/entities.dart';
import 'package:ambito/src/extensions/extensions.dart';
import 'package:ambito/src/packages/ambito_db/base_db.dart';
import 'package:isar/isar.dart';

import '../../../../main.dart';

class MeasureRepository extends BaseDB {
  @override
  IsarCollection collection = isar.measures;

  Future<bool> buildMeasureFilters() async {
    Map<String, List<int>> filtersAreaType = {};
    Map<String, List<int>> filtersMeasureGroup = {};
    Map<String, List<int>> filterMonths = {};
    Map<String, List<int>> filterFundingPrograms = {};
    Map<String, String> colors = {};

    var measures = getAll();

    for (Measure measure in measures) {
      measure.factsheetAreaType?.forEach((ivc) {
        filtersAreaType.putIfAbsent(ivc.value!, () => []).add(measure.id);
        colors[ivc.value!] = ivc.color!;
      });

      if (measure.actionGroup != null) {
        filtersMeasureGroup
            .putIfAbsent(measure.actionGroup!.value!, () => [])
            .add(measure.id);
        colors[measure.actionGroup!.value!] = measure.actionGroup!.color!;
      }

      measure.timeFrame?.forEach((ivc) {
        filterMonths.putIfAbsent(ivc.value!, () => []).add(measure.id);
        colors[ivc.value!] = ivc.color!;
      });

      measure.fundingPrograms?.forEach((ivc) {
        filterFundingPrograms.putIfAbsent(ivc.value!, () => []).add(measure.id);
      });
    }

    final itemFilterRepo = ItemFilterRepository();
    final autoIncrement = isar.itemFilters.autoIncrement;

    itemFilterRepo.putAll(filtersAreaType.entries
        .map((entry) => ItemFilter()
          ..id = autoIncrement()
          ..name = entry.key
          ..type = 'areaType'
          ..description =
              'Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt.'
          ..image =
              'images/actions/areatype/${entry.key.toLowerCase().replaceUmlauts()}.jpg'
          ..color = colors[entry.key]
          ..ids = entry.value)
        .toList());

    itemFilterRepo.putAll(filtersMeasureGroup.entries
        .map((entry) => ItemFilter()
          ..id = autoIncrement()
          ..name = entry.key
          ..type = 'group'
          ..description =
              'Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt.'
          ..image =
              'images/actions/areatype/${entry.key.toLowerCase().replaceUmlauts()}.jpg'
          ..color = colors[entry.key]
          ..ids = entry.value)
        .toList());

    itemFilterRepo.putAll(filterMonths.entries
        .map((entry) => ItemFilter()
          ..id = autoIncrement()
          ..name = entry.key
          ..type = 'month'
          ..ids = entry.value)
        .toList());

    itemFilterRepo.putAll(filterFundingPrograms.entries
        .map((entry) => ItemFilter()
          ..id = autoIncrement()
          ..name = entry.key
          ..type = 'fundingProgram'
          ..ids = entry.value)
        .toList());

    return true;
  }

  int getMeasureCount() {
    return isar.measures.where().count();
  }

  List<Measure?> getByIds(List<int> ids) {
    return isar.measures.getAll(ids);
  }

  List<Measure> getAllOrdered() {
    return isar.measures.where().sortByName().findAll();
  }
}
