import 'dart:convert';

import 'package:ambito/src/entity/entities.dart';
import 'package:ambito/src/packages/ambito_api/restclient.dart';
import 'package:flutter_dotenv/flutter_dotenv.dart';

import '../ambito_db/base_db.dart';

class BaseApi {
  Map tables = {};

  init() {
    // read table ids from .env file...
    tables = jsonDecode(dotenv.get('BASEROW_IDS'));
  }

  Future<bool> getContent(String table) async {
    init();
    int tableId = tables[table] ?? 0;

    if (tableId <= 0) return false;

    var response =
        await RestClient().get('$tableId/?user_field_names=true&size=200');
    if (response.statusCode != 200) return false;

    var json = _jsonDecoded(response.body);

    //if (table == 'measure') {
    //logger.d(json);
    //}

    var results = json['results'];

    final repositoryMap = {
      'tree_type': TreeTypeRepository().put,
      'measure': MeasureRepository().put,
      'measure_combination': MeasureCombinationRepository().put,
      'organism': OrganismRepository().put,
      'funding_program': FundingProgramRepository().put,
      'location_requirements': LocationRequirementsRepository().put,
      'reference_implementation': ReferenceImplementationRepository().put,
      'business': BusinessRepository().put,
      'service_provider': ServiceProviderRepository().put,
      'service_provider_contact_person':
          ServiceProviderContactPersonRepository().put,
      'material': MaterialRepository().put,
      'source': SourceRepository().put,
      'experience_report': ExperienceReportRepository().put,
      'approval_requirement': ApprovalRequirementRepository().put
    };

    var repositoryFunction = repositoryMap[table];
    if (repositoryFunction == null) return false;

    for (var item in results) {
      var model = _createModelFromJson(table, item);
      repositoryFunction(model);
    }

    return true;
  }

  dynamic _createModelFromJson(String table, Map<String, dynamic> json) {
    switch (table) {
      case 'tree_type':
        return TreeType.fromJson(json);
      case 'measure':
        return Measure.fromJson(json);
      case 'measure_combination':
        return MeasureCombination.fromJson(json);
      case 'organism':
        return Organism.fromJson(json);
      case 'funding_program':
        return FundingProgram.fromJson(json);
      case 'location_requirements':
        return LocationRequirements.fromJson(json);
      case 'reference_implementation':
        return ReferenceImplementation.fromJson(json);
      case 'business':
        return Business.fromJson(json);
      case 'service_provider':
        return ServiceProvider.fromJson(json);
      case 'service_provider_contact_person':
        return ServiceProviderContactPerson.fromJson(json);
      case 'material':
        return Material.fromJson(json);
      case 'source':
        return Source.fromJson(json);
      case 'experience_report':
        return ExperienceReport.fromJson(json);
      case 'approval_requirement':
        return ApprovalRequirement.fromJson(json);
      default:
        return null;
    }
  }

  dynamic _jsonDecoded(String input) {
    return jsonDecode(utf8.decode(input.runes.toList()));
  }
}
