import 'package:ambito/src/entity/entities.dart';
import 'package:flutter/foundation.dart';
import 'package:isar/isar.dart';
import 'package:path_provider/path_provider.dart';

import '../../../main.dart';

export 'repositories/_repositories.dart';

class AmbitoIsarDB {
  static init() async {
    await Isar.initialize();
    String dir = kIsWeb
        ? Isar.sqliteInMemory
        : (await getApplicationDocumentsDirectory()).path;
    const engine = kIsWeb ? IsarEngine.sqlite : IsarEngine.isar;
    isar = Isar.open(
      schemas: [
        ApprovalRequirementSchema,
        BusinessSchema,
        ExperienceReportSchema,
        FundingProgramSchema,
        ItemFilterSchema,
        LocationRequirementsSchema,
        MaterialSchema,
        MeasureSchema,
        MeasureCombinationSchema,
        NumberFactsheetSchema,
        OrganismSchema,
        ReferenceImplementationSchema,
        ServiceProviderSchema,
        ServiceProviderContactPersonSchema,
        SourceSchema,
        TreeTypeSchema,
      ],
      directory: dir,
      engine: engine,
      inspector: true,
    );
  }
}

abstract class BaseDB {
  abstract IsarCollection collection;

  put(BaseEntity entity) {
    isar.write((isar) {
      collection.put(entity);
    });
  }

  putAll(List<BaseEntity> entities) {
    isar.write((isar) {
      collection.putAll(entities);
    });
  }

  // get entity by id
  BaseEntity get(int id) {
    return collection.get(id);
  }

  // get all entities of selected collection
  List<dynamic> getAll() {
    return collection.where().findAll();
  }
}
