import 'package:ambito/src/entity/entities.dart';
import 'package:ambito/src/entity/measure/measure_repository.dart';
import 'package:ambito/src/pages/actions/cards/background_card.dart';
import 'package:ambito/src/pages/actions/cards/biodiverisity_card.dart';
import 'package:ambito/src/pages/actions/cards/creation_card.dart';
import 'package:ambito/src/pages/actions/cards/description_card.dart';
import 'package:ambito/src/pages/actions/cards/factsheet_card.dart';
import 'package:ambito/src/pages/actions/cards/funding_card.dart';
import 'package:ambito/src/pages/actions/cards/maintenance_card.dart';
import 'package:ambito/src/pages/actions/cards/presets_card.dart';
import 'package:ambito/src/pages/actions/cards/review_card.dart';
import 'package:flutter/material.dart';
import 'package:flutter_breadcrumb/flutter_breadcrumb.dart';
import 'package:get/get.dart';

import '../../../main.dart';
import '../../widgets/appbar/ambito_appbar.dart';
import '../ambito_page.dart';
import 'cards/advisor_card.dart';
import 'cards/material_card.dart';

class ActionDetailPage extends AmbitoPage {
  const ActionDetailPage({super.key});

  @override
  final String path = 'massnahme';
  @override
  final String title = 'Maßname';

  @override
  State<StatefulWidget> createState() => ActionDetailPageState();
}

class ActionDetailPageState extends State<ActionDetailPage> {
  late String id;

  Widget content = const SizedBox();
  Measure? massnahme;

  @override
  void initState() {
    id = Get.parameters['id'] ?? '';
    if (id != '') {
      massnahme = MeasureRepository().get(int.parse(id)) as Measure;
      setState(() {
        if (massnahme != null) {
          content = _buildInfoPage(massnahme!);
        }
      });
    }
    logger.d(id);
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AmbitoAppbar(
        links: const ['dashboard', 'massnahmen'],
      ),
      body: SingleChildScrollView(
        child: content,
      ),
    );
  }

  Widget _buildInfoPage(Measure massnahme) {
    return Column(
      children: [
        Align(
          alignment: Alignment.centerLeft,
          child: BreadCrumb(
            items: <BreadCrumbItem>[
              BreadCrumbItem(
                content: TextButton(
                  onPressed: () {
                    Get.offAndToNamed('/');
                  },
                  child: const Text('Start'),
                ),
              ),
              BreadCrumbItem(
                content: TextButton(
                  onPressed: () {
                    Get.offAndToNamed('/massnahmendatenbank');
                  },
                  child: const Text('Massnahmen'),
                ),
              ),
              BreadCrumbItem(
                content: Text(
                  massnahme.name ?? '',
                ),
              ),
            ],
            divider: const Icon(Icons.chevron_right),
          ),
        ),
        Container(
          width: double.infinity,
          height: 400,
          color: baseTheme.currentColorScheme.primary,
          child: massnahme.getImage(),
        ),
        SizedBox(
          width: 1140,
          child: Expanded(
            child: Column(
              children: [
                baseTheme.verticalSpacer,
                Align(
                  alignment: Alignment.centerLeft,
                  child: Text(
                    massnahme.name!,
                    style: baseTheme.currentThemeData.textTheme.titleLarge,
                  ),
                ),
                Row(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    Expanded(
                      child: Column(
                        children: [
                          DescriptionCard(
                            massnahme: massnahme,
                          ),
                          baseTheme.verticalSpacer,
                          BackgroundCard(
                            massnahme: massnahme,
                          ),
                          baseTheme.verticalSpacer,
                          PresetsCard(
                            massnahme: massnahme,
                          ),
                          baseTheme.verticalSpacer,
                          BiodiverisityCard(
                            massnahme: massnahme,
                          ),
                          baseTheme.verticalSpacer,
                          CreationCard(
                            massnahme: massnahme,
                          ),
                          baseTheme.verticalSpacer,
                          MaintenanceCard(
                            massnahme: massnahme,
                          ),
                          baseTheme.verticalSpacer,
                          FundingCard(
                            massnahme: massnahme,
                          ),
                          baseTheme.verticalSpacer,
                        ],
                      ),
                    ),
                    baseTheme.horizontalSpacer,
                    SizedBox(
                      width: 300,
                      child: Column(
                        mainAxisAlignment: MainAxisAlignment.start,
                        children: [
                          AdvisorCard(
                            massnahme: massnahme,
                          ),
                          baseTheme.verticalSpacer,
                          MaterialCard(
                            massnahme: massnahme,
                          ),
                          baseTheme.verticalSpacer,
                          FactsheetCard(
                            massnahme: massnahme,
                          ),
                          baseTheme.verticalSpacer,
                          ReviewCard(
                            massnahme: massnahme,
                          ),
                        ],
                      ),
                    ),
                  ],
                ),
              ],
            ),
          ),
        ),
      ],
    );
  }
}
