import 'package:ambito/src/entity/_general/filter/item_filter_repository.dart';
import 'package:ambito/src/entity/entities.dart';
import 'package:ambito/src/entity/measure/measure_repository.dart';
import 'package:ambito/src/pages/ambito_page.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

import '../../../main.dart';
import '../../packages/ambito_theme/ambito_theme.dart';
import '../../widgets/appbar/ambito_appbar.dart';

class ActionsPrePage extends AmbitoPage {
  const ActionsPrePage({super.key});

  @override
  final String path = 'massnahmen';
  @override
  final String title = 'Maßnamen';

  @override
  State<StatefulWidget> createState() => ActionsPrePageState();
}

class ActionsPrePageState extends State<ActionsPrePage> {
  List<Widget> cards = [];
  Map<String, int> counter = {};
  Map<String, int> ids = {};
  int counterComplete = 0;

  @override
  void initState() {
    List<ItemFilter>? filters = ItemFilterRepository().getByType('areaType');

    counterComplete = MeasureRepository().getMeasureCount();

    for (ItemFilter filter in filters!) {
      setState(() {
        cards.add(
          InkWell(
            onTap: () {
              prefs.setString('selected_areaType', filter.name!).then((value) {
                Get.toNamed('/massnahmendatenbank');
                setState(() {});
              });
            },
            onHover: (value) {},
            child: SizedBox(
              width: 300,
              height: 400,
              child: Card(
                color: actionAreaColors[filter.name!],
                child: Column(
                  children: [
                    ClipRRect(
                      borderRadius: BorderRadius.circular(8.0),
                      child: Image.asset(
                        filter.image!,
                      ),
                    ),
                    baseTheme.verticalSpacer,
                    Padding(
                      padding: const EdgeInsets.only(left: 16, right: 16),
                      child: Text(
                        filter.name!,
                        style: baseTheme.currentThemeData.textTheme.titleLarge
                            ?.copyWith(
                          color: actionAreaFGColors[filter.name!],
                          fontWeight: FontWeight.bold,
                        ),
                      ),
                    ),
                    baseTheme.verticalSpacer,
                    Padding(
                      padding: const EdgeInsets.only(left: 16, right: 16),
                      child: Text(
                        filter.description!,
                        style: baseTheme.currentThemeData.textTheme.bodyMedium
                            ?.copyWith(
                          color: baseTheme.currentColorScheme.onSurface,
                        ),
                      ),
                    ),
                    const Spacer(),
                    Padding(
                      padding: const EdgeInsets.only(
                          left: 16, right: 16, bottom: 16),
                      child: Text(
                        'Anzahl: ${filter.ids!.length}',
                        style: baseTheme.currentThemeData.textTheme.titleMedium
                            ?.copyWith(
                          color: actionAreaFGColors[filter.name!],
                          fontWeight: FontWeight.bold,
                        ),
                      ),
                    ),
                  ],
                ),
              ),
            ),
          ),
        );
      });
    }

    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AmbitoAppbar(
        links: const ['dashboard', 'massnahmen'],
      ),
      body: Center(
        child: Column(
          mainAxisSize: MainAxisSize.max,
          mainAxisAlignment: MainAxisAlignment.start,
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            baseTheme.verticalSpacerMax,
            Text(
              'Maßnahmenkategorien',
              textAlign: TextAlign.start,
              style:
                  baseTheme.currentThemeData.textTheme.headlineLarge?.copyWith(
                color: baseTheme.currentColorScheme.onSurface,
              ),
            ),
            baseTheme.verticalSpacerMax,
            Wrap(
              alignment: WrapAlignment.center,
              spacing: 16,
              children: cards,
            ),
            baseTheme.verticalSpacerMax,
            TextButton(
                onPressed: () async {
                  await prefs.setString('selected_areaType', '');
                  await Get.toNamed('/massnahmendatenbank');
                },
                child: Text(
                  'Zeig mir alle $counterComplete Maßnahmen...',
                  style: const TextStyle(
                    fontSize: 24,
                    fontWeight: FontWeight.bold,
                    color: Colors.blueAccent,
                  ),
                ))
          ],
        ),
      ),
    );
  }
}
