import 'package:ambito/src/extensions/extensions.dart';
import 'package:flutter/material.dart';

import '../../../../main.dart';
import '../../../entity/entities.dart';

class MaterialCard extends StatelessWidget {
  const MaterialCard({super.key, required this.massnahme});

  final Measure massnahme;

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      width: double.infinity,
      child: Card(
        shape: RoundedRectangleBorder(
          borderRadius: BorderRadius.circular(0),
        ),
        elevation: 0,
        color: baseTheme.currentColorScheme.tertiary,
        child: Padding(
          padding: const EdgeInsets.all(16),
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              Text(
                context.translate('page.actionDetailPage.material.title'),
                style: baseTheme.currentThemeData.textTheme.titleMedium,
              ),
              baseTheme.verticalSpacer,
              _getSortedMaterials(context),
              baseTheme.verticalSpacer,
              if (massnahme.costsMaterial != null)
                Text(context.translate(
                        'page.actionDetailPage.material.suggested_price') +
                    massnahme.costsMaterial +
                    ' €'),
              baseTheme.verticalSpacer,
            ],
          ),
        ),
      ),
    );
  }

  _getSortedMaterials(BuildContext context) {
    List<String> items = [];
    if (massnahme.materialAction != null) {
      for (IdValueColor ivc in massnahme.materialAction!) {
        items.add(ivc.value!);
      }
    }
    items.sort((a, b) => a.toString().compareTo(b.toString()));
    List<ListTile> tiles = [];
    for (String item in items) {
      tiles.add(
        ListTile(
          leading: Icon(Icons.fiber_manual_record),
          title: Text(
            item,
            style: Theme.of(context).textTheme.bodyMedium,
          ),
        ),
      );
    }
    return Column(
      children: tiles,
    );
  }
}
